#!D:\bld\gnuradio_1628795209068\_h_env\python.exe
#
# Copyright 2012, 2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
""" A tool for editing GNU Radio out-of-tree modules. """


from gnuradio.modtool.cli.base import cli


if __name__ == '__main__':
    try:
        cli()
    except KeyboardInterrupt:
        pass
